/**
 *    \file
 *    Defines function prototypes for using CAL.
 *    Copyright (C) 2004-2010 Nokia. All rights reserved.
 */
#ifndef CAL_H
#define CAL_H

/** Maximum name length for a block */
#define CAL_MAX_NAME_LEN	16
/** User area flag */
#define CAL_FLAG_USER		0x0001
/** Write-once area flag */
#define CAL_FLAG_WRITE_ONCE	0x0002

#ifdef __cplusplus
extern "C" {
#endif

struct cal;

/**
 * \brief Function pointer for libcal debug prints
 *
 * Create debug printing function and enable libcal debug messages by
 * pointing it to this.
 *
 * \param[in] level	Print level
 * \param[in] str	String to print
 */
extern void (*cal_debug_log) (int level, const char *str);

/**
 * \brief Function pointer for libcal error prints
 *
 * Create error printing function and enable libcal error messages by
 * pointing it to this.
 *
 * \param[in] str	Error String to print
 */
extern void (*cal_error_log) (const char *str);

/**
 * \brief Initialize CAL in read/write mode
 *
 * This prepares cal areas for use and locks MTD. Found CAL structure
 * is allocated and saved in given pointer. Use cal_finish() to close CAL and
 * to unlock MTD.
 *
 * \param[out]	cal_out Pointer to the CAL structure, use cal_finish() to free
 *
 * \retval 0	Success
 * \retval <0	Generic error
 */
int cal_init(struct cal **cal_out);

/**
 * \brief Initialize CAL in read-only mode
 *
 * This prepares cal areas for use and locks MTD. Found CAL structure
 * is allocated and saved in given pointer. Use cal_finish() to close CAL and
 * to unlock MTD.
 *
 * \param[out]	cal_out Pointer to the CAL structure, use cal_finish() to free
 *
 * \retval 0	Success
 * \retval <0	Generic error
 */
int cal_init_ro(struct cal **cal_out);

/**
 * \brief Closes and frees given CAL, unlocks MTD
 *
 * \param[in] cal	Pointer to the CAL structure
 */
void cal_finish(struct cal *cal);

/**
 * \brief Read specified block from CAL
 *
 * Reads newest occurrence of the block from CAL.
 *
 * \param[in] cal	Pointer to the CAL structure
 * \param[in] name	Block name
 * \param[out] ptr	Block data. Is allocated by using malloc() in
			libcal and caller should free it by using free()
 * \param[out] len	Length of block data
 * \param[in] flags	Area flag, use 0 as default (config area)
 *
 * \retval 0	Success
 * \retval -2	Block not found
 * \retval <0	Generic error
 */
int cal_read_block(struct cal *cal, const char *name,
		   void **ptr, unsigned long *len, unsigned long flags);

/**
 * \brief Write given block to CAL
 *
 * If writing fails, previous state is reverted.
 * cal_write_block() can also be used for deleting a block from CAL by writing
 * a zero length block.
 *
 * \param[in] cal	Pointer to the CAL structure
 * \param[in] name	Block name
 * \param[in] data	Block data
 * \param[in] data_len	Length of the data
 * \param[in] flags	Area flags, use 0 as default (config area)
 *
 * \retval 0	Success
 * \retval <0	Generic error
 */
int cal_write_block(struct cal *cal, const char *name,
		    const void *data, unsigned long data_len,
		    unsigned long flags);

/**
 * \brief Lock OneNAND OTP user region
 *
 * \param[in] cal	Pointer to the CAL structure
 * \param[in] flag	Area flag, use 0 as default (config area)
 *
 * \retval 0	Success
 * \retval <0	Generic error
 */
int cal_lock_otp_area(struct cal *cal, unsigned int flag);

#ifdef __cplusplus
}
#endif
#endif
